<?php

namespace Drupal\sso_connector\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\sso_connector\Services\JwtTokenValidator;
use Drupal\sso_connector\Services\UserSyncServiceInterface;
use Drupal\user\UserAuthInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for handling the return path from the Identity Provider.
 */
class SsoController extends ControllerBase {

  /**
   * The JWT token validator service.
   *
   * @var \Drupal\sso_connector\Services\JwtTokenValidator
   */
  protected $jwtTokenValidator;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The user authentication service.
   *
   * @var \Drupal\user\UserAuthInterface
   */
  protected $userAuth;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The user synchronization service.
   *
   * @var \Drupal\sso_connector\Services\UserSyncServiceInterface
   */
  protected $userSync;

  /**
   * Constructs a new SsoController.
   *
   * @param \Drupal\sso_connector\Services\JwtTokenValidator $jwt_token_validator
   *   The JWT token validator service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\user\UserAuthInterface $user_auth
   *   The user authentication service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Drupal\sso_connector\Services\UserSyncServiceInterface $user_sync
   *   The user synchronization service.
   */
  public function __construct(
    JwtTokenValidator $jwt_token_validator,
  EntityTypeManagerInterface $entity_type_manager,
  UserAuthInterface $user_auth,
  ConfigFactoryInterface $config_factory,
  UserSyncServiceInterface $user_sync
  ) {
    $this->jwtTokenValidator = $jwt_token_validator;
    $this->entityTypeManager = $entity_type_manager;
    $this->userAuth = $user_auth;
    $this->configFactory = $config_factory;
    $this->userSync = $user_sync;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('sso_connector.jwt_token_validator'),
      $container->get('entity_type.manager'),
      $container->get('user.auth'),
      $container->get('config.factory'),
      $container->get('sso_connector.user_sync')
    );
  }

  /**
   * Handles the return from the Identity Provider.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response to send back to the browser.
   */
  public function fromIdp(Request $request) {
    $config = $this->configFactory->get('sso_connector.settings');
    $idpUrl = $config->get('idp_url');
    $token = $request->query->get('token');

    if ($token) {
      $validationResult = $this->jwtTokenValidator->validate($token);
      if ($validationResult['valid']) {
        $userMail = $validationResult['data']['mail'];
        $users = $this->entityTypeManager->getStorage('user')
          ->loadByProperties(['mail' => $userMail]);
        $user = reset($users);
        if ($user) {
          user_login_finalize($user);
          return $this->redirect('<front>');
        }
        else {
          $userSync = $this->configFactory->get('sso_connector.settings')->get('user_sync');
          if($userSync) {
            $this->userSync->synchronizeUser($validationResult['data']);
          }
          $users = $this->entityTypeManager->getStorage('user')
            ->loadByProperties(['mail' => $userMail]);
          $user = reset($users);
          if ($user) {
            user_login_finalize($user);
            return $this->redirect('<front>');
          }
          else {
            return new TrustedRedirectResponse($idpUrl . '/?error=user_sync_error');
          }
        }
      }
      else {
        return new TrustedRedirectResponse($idpUrl . '/user/login?error=invalid_token');
      }
    }

    return new TrustedRedirectResponse($idpUrl . '/user/login');
  }

}
