<?php

namespace Drupal\sso_connector\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\sso_connector\Services\JwtTokenValidatorInterface;
use Drupal\sso_connector\Services\UserSyncServiceInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Handles Single Sign-On (SSO) session checks and redirections.
 *
 * This subscriber listens to kernel request events to manage Single Sign-On
 * (SSO) processes, including redirecting users from Service Providers (SP)
 * to the Identity Provider (IdP) for authentication and handling return
 * requests with JWT tokens.
 */
class CheckIdPSessionSubscriber implements EventSubscriberInterface {

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The JWT token validator service.
   *
   * @var \Drupal\sso_connector\Services\JwtTokenValidatorInterface
   */
  protected $jwtTokenValidator;

  /**
   * The private temporary store factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The synchronization users service.
   *
   * @var \Drupal\sso_connector\Services\UserSyncServiceInterface
   */
  protected $userSync;

  /**
   * The kill switch.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  protected $killSwitch;

  /**
   * Constructs a new CheckIdPSessionSubscriber object.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory service.
   * @param \Drupal\sso_connector\Services\JwtTokenValidatorInterface $jwtTokenValidator
   *   The JWT token validator service.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The private temporary store factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\sso_connector\Services\UserSyncServiceInterface $user_sync
   *   The synchronization users service.
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $killSwitch
   *   The kill switch.
   */
  public function __construct(
    AccountProxyInterface $currentUser,
    ConfigFactoryInterface $configFactory,
    JwtTokenValidatorInterface $jwtTokenValidator,
    PrivateTempStoreFactory $temp_store_factory,
    EntityTypeManagerInterface $entity_type_manager,
    UserSyncServiceInterface $user_sync,
    KillSwitch $killSwitch) {
    $this->currentUser = $currentUser;
    $this->configFactory = $configFactory;
    $this->jwtTokenValidator = $jwtTokenValidator;
    $this->tempStoreFactory = $temp_store_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->userSync = $user_sync;
    $this->killSwitch = $killSwitch;
  }

  /**
   * Responds to kernel request events.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The request event.
   */
  public function onKernelRequest(RequestEvent $event) {
    $request = $event->getRequest();
    $config = $this->configFactory->get('sso_connector.settings');
    $role = $config->get('role');
    $referer = $request->headers->get('referer');
    $allowedSpUrls = $config->get('sp_urls') ? explode("\n", $config->get('sp_urls')) : [];

    // Redirect from SP to IdP to start authentication.
    if ($role === 'sp') {
      $this->killSwitch->trigger();
      $idpUrl = $config->get('idp_url');
      if ($request->getPathInfo() === '/user/login' && $this->currentUser->isAnonymous()) {
        $event->setResponse(new TrustedRedirectResponse($idpUrl . '/return-path'));
      }
    }
    // Handle the return path from IdP after authentication.
    elseif ($role === 'idp') {
      $tempstore = $this->tempStoreFactory->get('sso_connector');
      $url_to_return = $tempstore->get('return_to_sp');
      if ($url_to_return) {
        $tempstore->delete('return_to_sp');
        $event->setResponse(new TrustedRedirectResponse($url_to_return));
      }
      if (str_contains($request->getPathInfo(), '/return-path') && in_array($referer, $allowedSpUrls)) {
        $this->handleSpReturn($event);
      }
    }
  }

  /**
   * Handles the return from an SP after authentication.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The request event.
   */
  protected function handleSpReturn(RequestEvent $event) {
    $request = $event->getRequest();
    $referer = $request->headers->get('referer');
    if ($this->currentUser->isAuthenticated()) {
      $config = $this->configFactory->get('sso_connector.settings');
      $allowedSpUrls = explode("\n", $config->get('sp_urls'));
      if (in_array($referer, $allowedSpUrls)) {
        $tempstore = $this->tempStoreFactory->get('sso_connector');
        $token = $tempstore->get('jwt_token');
        if ($token) {
          $validationResult = $this->jwtTokenValidator->validate($token);
          if ($validationResult['valid']) {
            $event->setResponse(new TrustedRedirectResponse($referer . '/from-idp?token=' . $token));
            return;
          }
          else {
            $event->setResponse(new RedirectResponse('/user/login?error=invalid_token&sp_to_return=' . $referer . 'user/login'));
            return;
          }
        }
        else {
          $event->setResponse(new RedirectResponse('/user/login'));
          return;
        }
      }
      else {
        $event->setResponse(new TrustedRedirectResponse($referer));
        return;
      }
    }
    else {
      $event->setResponse(new RedirectResponse('/user/login?error=invalid_token&sp_to_return=' . $referer));
      return;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['onKernelRequest', 299];
    return $events;
  }

}
