<?php

namespace Drupal\sso_connector\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\sso_connector\Services\JwtTokenValidatorInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Handles redirections to the SP after authentication at the IdP.
 */
class RedirectToSpSubscriber implements EventSubscriberInterface {
  /**
   * The JWT token validator service.
   *
   * @var \Drupal\sso_connector\Services\JwtTokenValidatorInterface
   */
  protected $jwtTokenValidator;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The private temporary store factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * Constructs a new RedirectToSpSubscriber object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory service.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $tempStoreFactory
   *   The private temporary store factory.
   * @param \Drupal\sso_connector\Services\JwtTokenValidatorInterface $jwtTokenValidator
   *   The JWT token validator service.
   */
  public function __construct(ConfigFactoryInterface $configFactory, PrivateTempStoreFactory $tempStoreFactory, JwtTokenValidatorInterface $jwtTokenValidator) {
    $this->configFactory = $configFactory;
    $this->tempStoreFactory = $tempStoreFactory;
    $this->jwtTokenValidator = $jwtTokenValidator;
  }

  /**
   * Responds to kernel response events.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The response event.
   */
  public function onKernelResponse(ResponseEvent $event) {
    $config = $this->configFactory->get('sso_connector.settings');
    $role = $config->get('role');
    if ($role === 'idp') {
      $tempstore = $this->tempStoreFactory->get('sso_connector');
      $urlToReturn = $tempstore->get('return_to_sp');
      if ($urlToReturn) {
        $tempstore->delete('return_to_sp');
        $tempstore = $this->tempStoreFactory->get('sso_connector');
        $token = $tempstore->get('jwt_token');
        if ($token) {
          $validationResult = $this->jwtTokenValidator->validate($token);
          if ($validationResult['valid']) {
            $event->setResponse(new TrustedRedirectResponse($urlToReturn . '/from-idp?token=' . $token));
            return;
          }
          else {
            $event->setResponse(new RedirectResponse('/user/login?error=invalid_token&sp_to_return=' . $urlToReturn . 'user/login'));
            return;
          }
        }
        else {
          $event->setResponse(new RedirectResponse('/user/login'));
          return;
        }
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents() {
    return [KernelEvents::RESPONSE => ['onKernelResponse', -10]];
  }

}
