<?php

namespace Drupal\sso_connector\EventSubscriber;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Session\SessionManagerInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\sso_connector\Event\UserLoginEvent;
use Drupal\sso_connector\Services\JwtTokenValidatorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Subscribes to user login events to generate a JWT token.
 *
 * This subscriber listens for user login events, generates a JWT token
 * for the authenticated user using the JwtTokenValidator, and stores the
 * token in a private temporary store for subsequent use.
 */
class UserLoginSubscriber implements EventSubscriberInterface, ContainerInjectionInterface {

  /**
   * The session manager service.
   *
   * @var \Drupal\Core\Session\SessionManagerInterface
   */
  protected $sessionManager;

  /**
   * The private temporary store factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The JWT token service.
   *
   * @var \Drupal\sso_connector\Services\JwtTokenValidatorInterface
   */
  protected $jwtTokenValidator;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new UserLoginSubscriber.
   *
   * @param \Drupal\Core\Session\SessionManagerInterface $session_manager
   *   The session manager service.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The private temporary store factory.
   * @param \Drupal\sso_connector\Services\JwtTokenValidatorInterface $jwt_token_service
   *   The JWT token service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(
    SessionManagerInterface $session_manager,
    PrivateTempStoreFactory $temp_store_factory,
    JwtTokenValidatorInterface $jwt_token_service,
    RequestStack $request_stack
    ) {
    $this->sessionManager = $session_manager;
    $this->tempStoreFactory = $temp_store_factory;
    $this->jwtTokenValidator = $jwt_token_service;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('session_manager'),
      $container->get('tempstore.private'),
      $container->get('sso_connector.jwt_token_validator'),
      $container->get('request_stack'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      UserLoginEvent::LOGIN => 'onUserLogin',
    ];
  }

  /**
   * Responds to user login events.
   *
   * Generates a JWT token for the logged-in user and stores it in the
   * private temporary store.
   *
   * @param \Drupal\sso_connector\Event\UserLoginEvent $event
   *   The user login event.
   */
  public function onUserLogin(UserLoginEvent $event) {
    $user = $event->getUser();
    $currentRequest = $this->requestStack->getCurrentRequest();
    $spToReturn = $currentRequest->query->get('sp_to_return');

    $userData = [];

    /** @var \Drupal\Core\Field\FieldItemListInterface[] $fieldItemList */
    foreach ($user->getFields() as $fieldName => $fieldItemList) {
      $values = [];
      foreach ($fieldItemList as $item) {
        $values[] = $item->value;
      }
      $userData[$fieldName] = count($values) === 1 ? reset($values) : $values;
    }

    $jwt = $this->jwtTokenValidator->generateTokenForUser($userData);

    $tempstore = $this->tempStoreFactory->get('sso_connector');
    $tempstore->set('jwt_token', $jwt);

    if ($spToReturn) {
      $tempstore->set('return_to_sp', $spToReturn);
    }
  }

}
