<?php

namespace Drupal\sso_connector\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form handler for the Identity Provider (IdP) configuration settings.
 *
 * This form allows administrators to configure settings related to the
 * Identity Provider, including roles, URLs, and JWT secret key.
 */
class IdpConfigForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['sso_connector.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sso_connector_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('sso_connector.settings');

    $form['role'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Is this the Identity Provider (IdP) site?'),
      '#default_value' => $config->get('role') == 'idp' ? TRUE : FALSE,
    ];

    $form['idp_url'] = [
      '#type' => 'url',
      '#title' => $this->t('Identity provider (IdP) Url:'),
      '#default_value' => $config->get('idp_url'),
      '#states' => [
        'invisible' => [
          ':input[name="role"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['sp_urls'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Service Providers (SPs) Urls:'),
      '#default_value' => $config->get('sp_urls'),
      '#states' => [
        'visible' => [
          ':input[name="role"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['secret_key_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'jwt-secret-key-wrapper'],
    ];

    $form['secret_key_wrapper']['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret Key'),
      '#description' => $this->t('secret key used to sign the tokens JWT. Same key should be used in IdP and SPs.'),
      '#default_value' => $config->get('secret_key'),
    ];

    $form['secret_key_wrapper']['generate'] = [
      '#type' => 'button',
      '#value' => $this->t('Generate secret key'),
      '#ajax' => [
        'callback' => '::generateSecretKeyAjax',
        'wrapper' => 'jwt-secret-key-wrapper',
        'effect' => 'fade',
      ],
      '#states' => [
        'visible' => [
          ':input[name="role"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['user_sync'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use User synchronization?'),
      '#description' => $this->t('By checking this checkbox, the users that exist in the IdP and not in the SPs will be created in these atomatically.'),
      '#default_value' => $config->get('user_sync'),
      '#states' => [
        'visible' => [
          ':input[name="role"]' => ['checked' => FALSE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('sso_connector.settings')
      ->set('role', $form_state->getValue('role') ? 'idp' : 'sp')
      ->set('idp_access', $form_state->getValue('idp_access') ? TRUE : FALSE)
      ->set('idp_url', $form_state->getValue('idp_url'))
      ->set('sp_urls', $form_state->getValue('role') ? $form_state->getValue('sp_urls') : '')
      ->set('secret_key', $form_state->getValue('secret_key'))
      ->set('user_sync', $form_state->getValue('user_sync'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    if ($form_state->getValue('role') == 'sp' && $form_state->getValue('sp_urls')) {
      $urls = explode("\n", $form_state->getValue('sp_urls'));
      foreach ($urls as $url) {
        if (!filter_var(trim($url), FILTER_VALIDATE_URL)) {
          $form_state->setErrorByName('sp_urls', $this->t('One or more URLs of the SPs are invalid.'));
        }
      }
    }
  }

  /**
   * AJAX callback to generate a new JWT secret key.
   *
   * This function generates a new random secret key to be used for signing
   * JWT tokens and updates the form element with the new value.
   *
   * @param array $form
   *   The form render array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array
   *   The render array for the JWT secret key form element.
   */
  public function generateSecretKeyAjax($form, $form_state) {
    $bytes = openssl_random_pseudo_bytes(32);
    $secretKey = bin2hex($bytes);
    $form_state->setValue(['secret_key_wrapper', 'secret_key'], $secretKey);
    $form_state->setRebuild(TRUE);
    $form['secret_key_wrapper']['secret_key']['#value'] = $secretKey;

    return $form['secret_key_wrapper'];
  }

}
