<?php

namespace Drupal\sso_connector\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Service for synchronizing user data between systems.
 *
 * This service provides functionality to synchronize user data from
 * an external source with the local Drupal user entities.
 */
class UserSyncService implements UserSyncServiceInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new UserSyncService object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory service.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, ConfigFactoryInterface $configFactory) {
    $this->entityTypeManager = $entityTypeManager;
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public function synchronizeUser(array $userData) {
    if (empty($userData['mail'])) {
      throw new \InvalidArgumentException('Email address is required for user synchronization.');
    }

    $user_storage = $this->entityTypeManager->getStorage('user');
    $users = $user_storage->loadByProperties(['mail' => $userData['mail']]);
    $user = reset($users);
    $userSync = $this->configFactory->get('sso_connector.settings')->get('user_sync');

    if (!$user && $userSync) {
      /** @var \Drupal\user\UserInterface $user */
      $user = $user_storage->create([
        'mail' => $userData['mail'],
        'status' => 1,
      ]);
    }
    else {
      return;
    }

    // Define fields that should not be synchronized.
    $excluded_fields = [
      'uid', 'uuid', 'langcode', 'preferred_langcode', 'preferred_admin_langcode',
      'mail', 'timezone', 'status', 'created', 'changed',
      'access', 'login', 'init', 'roles', 'default_langcode',
    ];

    // Update user entity with the provided data, excluding certain fields.
    foreach ($userData as $fieldName => $value) {
      if (!in_array($fieldName, $excluded_fields) && $user->hasField($fieldName)) {
        $user->set($fieldName, $value);
      }
    }

    $user->save();
  }

}
